@@ -1,7 +1,7 @@
 /*
  *  AudioOutput.h
  *
- *  Copyright (c) 2001-2012 Nick Dowell
+ *  Copyright (c) 2001-2015 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -22,20 +22,24 @@
 #ifndef _AUDIO_OUTPUT_H
 #define _AUDIO_OUTPUT_H
 
-#ifdef with_sndfile
-#include <sndfile.h>
-#endif
-
-#include "drivers/AudioInterface.h"
-#include "Config.h"
 #include "Thread.h"
 #include "main.h"
 #include "types.h"
 
+#ifdef with_sndfile
+#include <sndfile.h>
+#endif
+
+#include <string>
+
+using std::string;
+
+
 typedef void (* AudioCallback)(
 	float *buffer_l, float *buffer_r, unsigned num_frames, int stride,
 	amsynth_midi_event_t *events, unsigned event_count);
 
+
 class GenericOutput
 {
 public:
@@ -43,7 +47,7 @@ public:
 
 	virtual void		setAudioCallback(AudioCallback callback) { mAudioCallback = callback; }
 
-	virtual	int		init		( Config & config )	= 0;
+	virtual	int			init			() = 0;
 	
 	virtual	bool		Start 			() = 0;
 	virtual	void		Stop			() = 0;
@@ -76,15 +80,14 @@ public:
 	void 	stopRecording	( );
 	void 	setOutputFile	( string file )	{ wavoutfile = file; };
   	string 	getOutputFile	( )	{ return wavoutfile; };
-	int 	init		( Config & config );
+	int 	init			( );
 
 	void	ThreadAction	();
 
 private:
   int running;
   int channels;
-  Config *config;
-  AudioInterface out;
+  class AudioDriver *driver;
   string wavoutfile;
   int recording;
   float	*buffer;
@@ -95,7 +98,7 @@ private:
 };
 
 class NullAudioOutput : public GenericOutput { public:
-	virtual	int  init  ( Config & ) { return -1; }
+	virtual	int  init  () { return -1; }
 	virtual	bool Start () { return -1; }
 	virtual	void Stop  () {}
 };
