@@ -23,12 +23,11 @@
 #define _ConfigDialog_h
 
 #include <gtkmm.h>
-#include "../Config.h"
 
 class ConfigDialog : public Gtk::Dialog
 {
 public:
-	ConfigDialog (Gtk::Window& parent, Config &);
+	ConfigDialog (Gtk::Window& parent);
 	
 protected:
 	void ReadValues ();
@@ -36,7 +35,6 @@ protected:
 	virtual void on_response (int);
 
 private:
-	Config &		mConfig;
 	Gtk::ComboBoxText	mMidiDriver;
 	Gtk::ComboBoxText	mAudioDriver;
 	Gtk::ComboBoxText	mSampleRate;
