@@ -19,10 +19,16 @@
  *  You should have received a copy of the GNU General Public License
  *  along with amsynth.  If not, see <http://www.gnu.org/licenses/>.
  */
+
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
  
 #include "GUI.h"
 
 #include <assert.h>
+#include <iostream>
+#include <sstream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <list>
@@ -38,20 +44,22 @@
 #endif
 
 using namespace Gtk;
+using std::cout;
+using std::cerr;
+using std::endl;
+using std::ostringstream;
 
 #include "../AudioOutput.h"
+#include "../Configuration.h"
 #include "../MidiController.h"
 #include "../Preset.h"
 #include "../Synthesizer.h"
-
-#include "../../config.h"
 #include "ConfigDialog.h"
+#include "editor_pane.h"
+#include "gui_main.h"
 #include "MIDILearnDialog.h"
 #include "PresetControllerView.h"
 
-#include "gui_main.h"
-#include "editor_pane.h"
-
 enum {
 	evLoad,
 	evCommit,
@@ -119,11 +127,11 @@ GUI::delete_event_impl(GdkEventAny *)
 	return true;
 }
 
-GUI::GUI( Config & config_in, MidiController & mc, Synthesizer *synth, GenericOutput *audio )
+GUI::GUI(MidiController & mc, Synthesizer *synth, GenericOutput *audio)
 :	m_auditionKeyDown(false)
 ,	m_synth(synth)
+,	m_presetIsNotSaved(false)
 {
-	this->config = &config_in;
 	this->midi_controller = &mc;
 	this->audio_out = audio;
 	
@@ -210,6 +218,8 @@ GUI::create_menus	( )
 	using namespace Gtk::Menu_Helpers;
 	using namespace Gtk;
 	using sigc::bind;
+
+	Configuration & config = Configuration::get();
 	
 	//
 	// File menu
@@ -322,7 +332,7 @@ GUI::create_menus	( )
 			name << i;
 			name << " Semitones";
 			Gtk::RadioMenuItem *item = Gtk::manage(new Gtk::RadioMenuItem(grp, name.str()));
-			item->set_active(i == config->pitch_bend_range);
+			item->set_active(i == config.pitch_bend_range);
 			item->signal_activate().connect( sigc::bind(mem_fun(*this, &GUI::on_pitch_bend_range_change), i, item) );
 			m_pitchBendRangeMenu->items().push_back(*item);
 		}
@@ -344,7 +354,7 @@ GUI::create_menus	( )
 	MenuItem *menu_item = manage (new MenuItem("Virtual Keyboard"));
 	menu_item->signal_activate().connect(sigc::bind(mem_fun(*this, &GUI::event_handler),(int)evVkeybd));
 	// vkeybd must exist, and we must be using ALSA MIDI
-	if (config->alsa_seq_client_id == 0 || command_exists("vkeybd") != 0)
+	if (config.alsa_seq_client_id == 0 || command_exists("vkeybd") != 0)
 		menu_item->set_sensitive( false );
 	list_utils.push_back (*menu_item);
 
@@ -364,13 +374,13 @@ GUI::create_menus	( )
 	menu_item = manage (new MenuItem("kaconnect"));
 	menu_item->signal_activate().connect(sigc::bind(mem_fun(*this, &GUI::command_run),"kaconnect"));
 	if (command_exists ("kaconnect") != 0) menu_item->set_sensitive( false );
-	if (config->alsa_seq_client_id==0) menu_item->set_sensitive( false );
+	if (config.alsa_seq_client_id==0) menu_item->set_sensitive( false );
 	list_utils_midi.push_back (*menu_item);
 	
 	menu_item = manage (new MenuItem("alsa-patch-bay"));
 	menu_item->signal_activate().connect (sigc::bind(mem_fun(*this, &GUI::command_run),"alsa-patch-bay --driver alsa"));
 	if (command_exists ("alsa-patch-bay") != 0) menu_item->set_sensitive( false );
-	if (config->alsa_seq_client_id==0) menu_item->set_sensitive( false );
+	if (config.alsa_seq_client_id==0) menu_item->set_sensitive( false );
 	list_utils_midi.push_back (*menu_item);
 	
 	list_utils.push_back (MenuElem("MIDI (ALSA) connections", *menu_utils_midi));
@@ -384,13 +394,13 @@ GUI::create_menus	( )
 	menu_item = manage (new MenuItem("qjackconnect"));
 	menu_item->signal_activate().connect (sigc::bind(mem_fun(*this, &GUI::command_run),"qjackconnect"));
 	if (command_exists ("qjackconnect") != 0) menu_item->set_sensitive( false );
-	if (config->current_audio_driver != "jack" && config->current_audio_driver != "JACK") menu_item->set_sensitive( false );
+	if (config.current_audio_driver != "jack" && config.current_audio_driver != "JACK") menu_item->set_sensitive( false );
 	list_utils_jack.push_back (*menu_item);
 	
 	menu_item = manage (new MenuItem("alsa-patch-bay"));
 	menu_item->signal_activate().connect (sigc::bind(mem_fun(*this, &GUI::command_run),"alsa-patch-bay --driver jack"));
 	if (command_exists ("alsa-patch-bay") != 0) menu_item->set_sensitive( false );
-	if (config->current_audio_driver != "jack" && config->current_audio_driver != "JACK") menu_item->set_sensitive( false );
+	if (config.current_audio_driver != "jack" && config.current_audio_driver != "JACK") menu_item->set_sensitive( false );
 	list_utils_jack.push_back (*menu_item);
 	
 	list_utils.push_back (MenuElem("Audio (JACK) connections", *menu_utils_jack));
@@ -418,7 +428,7 @@ GUI::create_menus	( )
 	list_bar.push_back (MenuElem("_Utils", Gtk::AccelKey("<alt>U"), *menu_utils));
 	list_bar.push_back (MenuElem("_Help", *menu_help));
     
-    gchar *text = g_strdup_printf ("Audio: %s @ %d  MIDI: %s", config->current_audio_driver.c_str(), config->sample_rate, config->current_midi_driver.c_str());
+    gchar *text = g_strdup_printf ("Audio: %s @ %d  MIDI: %s", config.current_audio_driver.c_str(), config.sample_rate, config.current_midi_driver.c_str());
     list_bar.push_back (MenuElem (text));
     list_bar.back().set_right_justified();
     list_bar.back().set_sensitive(false);
@@ -447,19 +457,6 @@ GUI::init()
 {
 	Preset *preset = &(preset_controller->getCurrentPreset());
 	
-	// Add custom signal for atomic change operations to parameters.
-	g_signal_new(
-		"start_atomic_value_change",
-		g_type_from_name("GtkAdjustment"),
-		G_SIGNAL_ACTION,
-		0,
-		NULL,
-		NULL,
-		NULL,
-		G_TYPE_NONE,
-		0
-	);
-	
 	for (int i=0; i<kAmsynthParameterCount; i++) {
 		Parameter &param = preset->getParameter(i);
 		m_adjustments[i] = (GtkAdjustment *) gtk_adjustment_new (
@@ -468,23 +465,24 @@ GUI::init()
 			param.getMax(),
 			param.getStep(),
 			0, 0);
-		gtk_signal_connect (GTK_OBJECT (m_adjustments[i]), "value_changed",
-			(GtkSignalFunc) adjustment_value_changed,
-			(gpointer) &param );
-
-		m_undoArgs[i] = new UndoArgs(preset_controller, &param);
-		g_signal_connect_after (G_OBJECT (m_adjustments[i]), "start_atomic_value_change",
-			G_CALLBACK(start_atomic_adjustment_value_change),
-			(gpointer) m_undoArgs[i] );
 	}
 	
 	Gtk::Widget *editor = Glib::wrap (editor_pane_new (m_adjustments, FALSE));
 	
+	// start_atomic_value_change is not registered until editor_pane_new is called
+	for (int i=0; i<kAmsynthParameterCount; i++) {
+		Parameter &param = preset->getParameter(i);
+		m_undoArgs[i] = new UndoArgs(preset_controller, &param);
+		g_signal_connect_after (G_OBJECT (m_adjustments[i]), "start_atomic_value_change",
+			G_CALLBACK(start_atomic_adjustment_value_change),
+			(gpointer) m_undoArgs[i] );
+		gtk_signal_connect (GTK_OBJECT (m_adjustments[i]), "value_changed",
+			(GtkSignalFunc) adjustment_value_changed,
+			(gpointer) &param );
+	}
+	
 	vbox.pack_start (*(create_menus ()),0,0);
-	Gtk::HBox *tmphbox = manage (new Gtk::HBox());
-	tmphbox->pack_start(*presetCV,0,0);
-
-	vbox.pack_start (*tmphbox,0,0);
+	vbox.pack_start (*presetCV, false, false);
 	vbox.pack_start (*editor, Gtk::PACK_EXPAND_WIDGET,0);
 	add(vbox);
 
@@ -501,25 +499,26 @@ GUI::init()
 void
 GUI::post_init()
 {
+	Configuration & config = Configuration::get();
 	bool bad_config = false;
 	
-	if (config->current_audio_driver.empty())
+	if (config.current_audio_driver.empty())
 	{
 		bad_config = true;
-		MessageDialog dlg (*this, "amSynth configuration error", false, MESSAGE_ERROR, BUTTONS_OK, true);
+		MessageDialog dlg (*this, "amsynth configuration error", false, MESSAGE_ERROR, BUTTONS_OK, true);
 		dlg.set_secondary_text(
-			"amSynth could not initialise the selected audio device.\n\n"
+			"amsynth could not initialise the selected audio device.\n\n"
 			"Please review the configuration and restart"
 		    );
 		dlg.run();
 	}
 	
-	if (config->current_midi_driver.empty())
+	if (config.current_midi_driver.empty())
 	{
 		bad_config = true;
-		MessageDialog dlg (*this, "amSynth configuration error", false, MESSAGE_ERROR, BUTTONS_OK, true);
+		MessageDialog dlg (*this, "amsynth configuration error", false, MESSAGE_ERROR, BUTTONS_OK, true);
 		dlg.set_secondary_text(
-			"amSynth could not initialise the selected midi device.\n\n"
+			"amsynth could not initialise the selected midi device.\n\n"
 			"Please review the configuration and restart"
 		    );
 		dlg.run();
@@ -534,10 +533,10 @@ GUI::post_init()
 	
 #ifdef ENABLE_REALTIME
 	// show realtime warning message if necessary
-	if (config->current_audio_driver_wants_realtime == 1 &&
-		config->realtime == 0)
+	if (config.current_audio_driver_wants_realtime == 1 &&
+		config.realtime == 0)
 	{
-		MessageDialog dlg (*this, "amSynth could not set realtime priority");
+		MessageDialog dlg (*this, "amsynth could not set realtime priority");
 		dlg.set_secondary_text ("You may experience audio buffer underruns resulting in 'clicks' in the audio.\n\nThis is most likely because the program is not SUID root.\n\nUsing the JACK audio subsystem can also help");
 		dlg.run();
 	}
@@ -697,7 +696,8 @@ GUI::event_handler(const int e)
 	case evVkeybd:
 	    {
 			char tmp[255] = "";
-			snprintf(tmp, sizeof(tmp), "vkeybd --addr %d:0", config->alsa_seq_client_id);
+			Configuration & config = Configuration::get();
+			snprintf(tmp, sizeof(tmp), "vkeybd --addr %d:0", config.alsa_seq_client_id);
 			command_run(tmp);
 	    }
 		break;
@@ -711,7 +711,7 @@ GUI::event_handler(const int e)
 		
 	case evConfig:
 	{
-		ConfigDialog dlg (*this, *config);
+		ConfigDialog dlg (*this);
 		dlg.run ();
 		break;
 	}
@@ -740,23 +740,24 @@ GUI::event_handler(const int e)
             "Samuli Suominen",
             NULL
         };
+        std::string version = VERSION;
         gtk_show_about_dialog(this->gobj(),
                               "program-name", PACKAGE,
-                              "version", VERSION,
+                              "version", version.c_str(),
                               "authors", authors,
                               "comments", "Analogue Modelling SYNTHesizer",
-                              "website", "http://code.google.com/p/amsynth/",
-                              "copyright", "© 2002 - 2014 Nick Dowell and contributors",
+                              "website", PACKAGE_URL,
+                              "copyright", "© 2002 - 2015 Nick Dowell and contributors",
                               NULL);
         break;
     }
 
 	case evHelpMenuBugReport:
-		open_uri("http://code.google.com/p/amsynth/issues/list");
+		open_uri(PACKAGE_BUGREPORT);
 		break;
 
 	case evHelpMenuOnlineDocumentation:
-		open_uri("http://code.google.com/p/amsynth/w/list");
+		open_uri("https://github.com/amsynth/amsynth/wiki");
 		break;
 
 	default:
@@ -787,12 +788,13 @@ GUI::onUpdate()	// called whenever the preset selection has changed
 void
 GUI::update_title()
 {
+	Configuration & config = Configuration::get();
 	std::ostringstream ostr;
 	ostr << "amsynth";
 
-	if (config->jack_client_name.length() && config->jack_client_name != "amsynth") {
+	if (config.jack_client_name.length() && config.jack_client_name != "amsynth") {
 		ostr << ": ";
-		ostr << config->jack_client_name;
+		ostr << config.jack_client_name;
 	}
 
 	ostr << ": ";
@@ -890,7 +892,7 @@ GUI::preset_paste_callback(GtkClipboard *, const gchar *text, gpointer userdata)
 	GUI *_this = reinterpret_cast<GUI *>(userdata);
 	
 	Preset pastedPreset;
-	if (!pastedPreset.fromString(std::string(text)))
+	if (!text || !pastedPreset.fromString(std::string(text)))
 		return;
 	
 	// enure preset has a unique name
@@ -921,11 +923,12 @@ GUI::preset_paste_as_new_callback(GtkClipboard *clipboard, const gchar *text, gp
 void
 GUI::bank_open		( )
 {
+	Configuration & config = Configuration::get();
 	std::string filename = file_dialog(this->gobj(), "Open Bank", false, NULL, NULL, NULL);
 	if (!filename.empty()) {
-		preset_controller->savePresets(config->current_bank_file.c_str());
-		config->current_bank_file = filename;
-		preset_controller->loadPresets(config->current_bank_file.c_str());
+		preset_controller->savePresets(config.current_bank_file.c_str());
+		config.current_bank_file = filename;
+		preset_controller->loadPresets(config.current_bank_file.c_str());
 	}
 }
 
@@ -1000,20 +1003,20 @@ GUI::tuning_reset	( )
 int
 GUI::command_exists	(const char *command)
 {
-	std::string cmdline = "which " + std::string(command) + " > /dev/null";
-	int result = system(cmdline.c_str());
-	return result;
+	gchar *argv[] = { (gchar *)"/usr/bin/which", (gchar *)command, NULL };
+	gint exit_status = 0;
+	if (g_spawn_sync(NULL, argv, NULL,
+		(GSpawnFlags)(G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL),
+		NULL, NULL, NULL, NULL, &exit_status, NULL))
+		return exit_status;
+	return -1;
 }
 
 void
 GUI::command_run	(const char *command)
 {
-	// TODO: would be better to fork() and exec()
-	
 	string full_command = std::string(command) + std::string(" &");
-	// returns 0 even if command could not be run
-	int result = system(full_command.c_str());
-	result = 0;
+	system(full_command.c_str());
 }
 
 void
@@ -1025,10 +1028,11 @@ GUI::on_midi_channel_change(int value)
 void
 GUI::on_ployphony_change(int value, Gtk::RadioMenuItem *item)
 {
+	Configuration & config = Configuration::get();
 	if (item->get_active()) {
-		if (config->polyphony != value) {
-			config->polyphony = value;
-			config->save();
+		if (config.polyphony != value) {
+			config.polyphony = value;
+			config.save();
 		}
 		m_synth->setMaxNumVoices(value);
 	}
@@ -1046,11 +1050,12 @@ GUI::on_pitch_bend_range_menu_show()
 void
 GUI::on_pitch_bend_range_change(int value, Gtk::RadioMenuItem *item)
 {
+	Configuration & config = Configuration::get();
 	if (item->get_active()) {
-		if (config->pitch_bend_range != value) {
-			config->pitch_bend_range = value;
-			config->save();
-			m_synth->setPitchBendRangeSemitones(config->pitch_bend_range);
+		if (config.pitch_bend_range != value) {
+			config.pitch_bend_range = value;
+			config.save();
+			m_synth->setPitchBendRangeSemitones(config.pitch_bend_range);
 		}
 	}
 }
