@@ -23,7 +23,7 @@
 #define _JACK_OUTPUT_H
 
 #if HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #ifdef WITH_JACK
@@ -31,7 +31,6 @@
 #endif
 
 #include "AudioOutput.h"
-#include "Config.h"
 
 class JackOutput : public GenericOutput {
 
@@ -39,7 +38,7 @@ public:
 
 	JackOutput();
 	
-	int			init		( Config & config );// returns 0 on success
+	int			init		(); // returns 0 on success
 	bool		Start		();
 	void		Stop		();
 	
