@@ -21,38 +21,37 @@
 
 #include "Synthesizer.h"
 
+#include "Configuration.h"
 #include "MidiController.h"
 #include "PresetController.h"
 #include "VoiceAllocationUnit.h"
 #include "VoiceBoard/VoiceBoard.h"
 
+#include <algorithm>
 #include <cassert>
 #include <cstdio>
+#include <cstring>
 
 
-Synthesizer::Synthesizer(Config *config)
-: _sampleRate(config ? config->sample_rate : 44100)
+Synthesizer::Synthesizer()
+: _sampleRate(-1)
 , _midiController(0)
 , _presetController(0)
 , _voiceAllocationUnit(0)
 {
-	if (!config) {
-		config = new Config;
-		config->Defaults();
-		config->load();
-	}
+	Configuration &config = Configuration::get();
 
 	_voiceAllocationUnit = new VoiceAllocationUnit;
 	_voiceAllocationUnit->SetSampleRate(_sampleRate);
-	_voiceAllocationUnit->SetMaxVoices(config->polyphony);
-	_voiceAllocationUnit->setPitchBendRangeSemitones(config->pitch_bend_range);
+	_voiceAllocationUnit->SetMaxVoices(config.polyphony);
+	_voiceAllocationUnit->setPitchBendRangeSemitones(config.pitch_bend_range);
 	
 	_presetController = new PresetController;
-	_presetController->loadPresets(config->current_bank_file.c_str());
+	_presetController->loadPresets(config.current_bank_file.c_str());
 	_presetController->selectPreset(0);
 	_presetController->getCurrentPreset().AddListenerToAll(_voiceAllocationUnit);
 	
-	_midiController = new MidiController(*config);
+	_midiController = new MidiController();
 	_midiController->SetMidiEventHandler(_voiceAllocationUnit);
 	_midiController->setPresetController(*_presetController);
 }
@@ -76,6 +75,17 @@ void Synthesizer::saveBank(const char *filename)
 	_presetController->savePresets(filename);
 }
 
+void Synthesizer::loadState(char *buffer)
+{
+	_presetController->getCurrentPreset().fromString(buffer);
+}
+
+int Synthesizer::saveState(char **buffer)
+{
+	std::string string = _presetController->getCurrentPreset().toString();
+	return asprintf(buffer, "%s", string.c_str());
+}
+
 const char *Synthesizer::getPresetName(int presetNumber)
 {
 	return _presetController->getPreset(presetNumber).getName().c_str();
@@ -96,11 +106,36 @@ float Synthesizer::getParameterValue(Param parameter)
 	return _presetController->getCurrentPreset().getParameter(parameter).getValue();
 }
 
+float Synthesizer::getNormalizedParameterValue(Param parameter)
+{
+	return _presetController->getCurrentPreset().getParameter(parameter).GetNormalisedValue();
+}
+
 void Synthesizer::setParameterValue(Param parameter, float value)
 {
 	_presetController->getCurrentPreset().getParameter(parameter).setValue(value);
 }
 
+void Synthesizer::setNormalizedParameterValue(Param parameter, float value)
+{
+	_presetController->getCurrentPreset().getParameter(parameter).SetNormalisedValue(value);
+}
+
+void Synthesizer::getParameterName(Param parameter, char *buffer, size_t maxLen)
+{
+	strncpy(buffer, _presetController->getCurrentPreset().getParameter(parameter).getName().c_str(), maxLen);
+}
+
+void Synthesizer::getParameterLabel(Param parameter, char *buffer, size_t maxLen)
+{
+	strncpy(buffer, _presetController->getCurrentPreset().getParameter(parameter).getLabel().c_str(), maxLen);
+}
+
+void Synthesizer::getParameterDisplay(Param parameter, char *buffer, size_t maxLen)
+{
+	strncpy(buffer, _presetController->getCurrentPreset().getParameter(parameter).GetStringValue().c_str(), maxLen);
+}
+
 int Synthesizer::getPitchBendRangeSemitones()
 {
 	return _voiceAllocationUnit->getPitchBendRangeSemitones();
@@ -142,8 +177,15 @@ void Synthesizer::setSampleRate(int sampleRate)
 	_voiceAllocationUnit->SetSampleRate(sampleRate);
 }
 
-void Synthesizer::process(unsigned int nframes, const std::vector<amsynth_midi_event_t> &midi_in, float *audio_l, float *audio_r, unsigned audio_stride)
+static bool comapare(const amsynth_midi_event_t &first, const amsynth_midi_event_t &second) { return (first.offset_frames < second.offset_frames); }
+
+void Synthesizer::process(unsigned int nframes, std::vector<amsynth_midi_event_t> &midi_in, float *audio_l, float *audio_r, unsigned audio_stride)
 {
+	if (_sampleRate < 0) {
+		assert(!"sample rate has not been set");
+		return;
+	}
+	std::sort(midi_in.begin(), midi_in.end(), comapare);
 	std::vector<amsynth_midi_event_t>::const_iterator event = midi_in.begin();
 	unsigned frames_left_in_buffer = nframes, frame_index = 0;
 	while (frames_left_in_buffer) {
@@ -155,7 +197,6 @@ void Synthesizer::process(unsigned int nframes, const std::vector<amsynth_midi_e
 		unsigned block_size_frames = std::min(frames_left_in_buffer, (unsigned)VoiceBoard::kMaxProcessBufferSize);
 		if (event != midi_in.end() && event->offset_frames > frame_index) {
 			unsigned frames_until_next_event = event->offset_frames - frame_index;
-			assert(frames_until_next_event < frames_left_in_buffer);
 			block_size_frames = std::min(block_size_frames, frames_until_next_event);
 		}
 		
@@ -166,4 +207,8 @@ void Synthesizer::process(unsigned int nframes, const std::vector<amsynth_midi_e
 		frame_index += block_size_frames;
 		frames_left_in_buffer -= block_size_frames;
 	}
+	while (event != midi_in.end()) {
+		_midiController->HandleMidiData(event->buffer, event->length);
+		++event;
+	}
 }
