@@ -27,7 +27,6 @@
 
 #include <vector>
 
-class Config;
 class MidiController;
 class PresetController;
 class VoiceAllocationUnit;
@@ -36,12 +35,15 @@ class Synthesizer
 {
 public:
     
-    Synthesizer(Config *config = 0);
+    Synthesizer();
     ~Synthesizer();
     
     void loadBank(const char *filename);
     void saveBank(const char *filename);
-    
+
+    void loadState(char *buffer);
+    int saveState(char **buffer);
+
     const char *getPresetName(int presetNumber);
 
     int getPresetNumber();
@@ -50,6 +52,13 @@ public:
     float getParameterValue(Param parameter);
     void setParameterValue(Param parameter, float value);
 
+    float getNormalizedParameterValue(Param parameter);
+    void setNormalizedParameterValue(Param parameter, float value);
+
+    void getParameterName(Param parameter, char *buffer, size_t maxLen);
+    void getParameterLabel(Param parameter, char *buffer, size_t maxLen);
+    void getParameterDisplay(Param parameter, char *buffer, size_t maxLen);
+
 	int getPitchBendRangeSemitones();
 	void setPitchBendRangeSemitones(int value);
 
@@ -63,7 +72,7 @@ public:
 	void setSampleRate(int sampleRate);
 
     void process(unsigned nframes,
-                 const std::vector<amsynth_midi_event_t> &midi_in,
+                 std::vector<amsynth_midi_event_t> &midi_in,
                  float *audio_l, float *audio_r,
                  unsigned audio_stride = 1);
     
